>From beaba08afc645dc283a7c79c7b0430f9293b3229 Mon Sep 17 00:00:00 2001
From: =?utf-8?q?Cyril=20Math=C3=A9?= <cmathe@actech-innovation.com>
Date: Tue, 23 Jun 2009 09:46:00 +0200
Subject: [PATCH] libvlc API: Add Marquee Filter to libvlc in video.c

  -libvlc_video_get_marq_option_as_int    : get an option int value
  -libvlc_video_get_marq_option_as_string : get an option string value
  -libvlc_video_set_marq_option_as_int    : enable, disable or set an int value
  -libvlc_video_set_marq_option_as_string : set a string value to marq
---
 include/vlc/libvlc_media_player.h |   63 +++++++
 src/control/video.c               |  338 +++++++++++++++++++++++++++++++++++++
 src/libvlc.sym                    |    4 +
 3 files changed, 405 insertions(+), 0 deletions(-)

diff --git a/include/vlc/libvlc_media_player.h b/include/vlc/libvlc_media_player.h
index 4b811a0..ca76119 100644
--- a/include/vlc/libvlc_media_player.h
+++ b/include/vlc/libvlc_media_player.h
@@ -76,6 +76,23 @@ typedef struct libvlc_rectangle_t
     int bottom, right;
 } libvlc_rectangle_t;
 
+/**
+ * Marq options definition
+ */
+
+typedef enum libvlc_video_marq_option_t {
+    libvlc_marq_Error       = -1,
+    libvlc_marq_Enabled     =  1,
+    libvlc_marq_Color       =  2,
+    libvlc_marq_Opacity     =  3,
+    libvlc_marq_Position    =  4,
+    libvlc_marq_Refresh     =  5,
+    libvlc_marq_Size        =  6,
+    libvlc_marq_Text        =  7,
+    libvlc_marq_Timeout     =  8,
+    libvlc_marq_X           =  9,
+    libvlc_marq_Y           =  10
+} libvlc_video_marq_option_t;
 
 /**
  * Create an empty Media Player object
@@ -726,6 +743,52 @@ VLC_PUBLIC_API void libvlc_video_set_deinterlace( libvlc_media_player_t *,
                                                   int , const char *,
                                                   libvlc_exception_t *);
 
+/**
+ * Get an option value (option which return an int)
+ *
+ * \param p_mi libvlc media player
+ * \param option marq option to get
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API int libvlc_video_get_marq_option_as_int( libvlc_media_player_t *,
+                                                        libvlc_video_marq_option_t,
+                                                        libvlc_exception_t * );
+
+/**
+ * Get an option value (option which return a string)
+ *
+ * \param p_mi libvlc media player
+ * \param option marq option to get
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API char *libvlc_video_get_marq_option_as_string( libvlc_media_player_t *,
+                                                             libvlc_video_marq_option_t,
+                                                             libvlc_exception_t * );
+
+/**
+ * Enable, disable or set a marq option (only int)
+ *
+ * \param p_mi libvlc media player
+ * \param option marq option to set
+ * \param i_val marq option value
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_marq_option_as_int( libvlc_media_player_t *,
+                                                         libvlc_video_marq_option_t,
+                                                         int, libvlc_exception_t * );
+
+/**
+ * Set a marq option (only string)
+ *
+ * \param p_mi libvlc media player
+ * \param option marq option to set
+ * \param psz_text marq option value
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_marq_option_as_string( libvlc_media_player_t *,
+                                                            libvlc_video_marq_option_t,
+                                                            const char *,
+                                                            libvlc_exception_t * );
 /** @} video */
 
 /** \defgroup libvlc_audio libvlc_audio
diff --git a/src/control/video.c b/src/control/video.c
index d90b1a4..19f0d2f 100644
--- a/src/control/video.c
+++ b/src/control/video.c
@@ -34,6 +34,7 @@
 #include <vlc_vout.h>
 
 #include "media_player_internal.h"
+#include <vlc_osd.h>
 
 /*
  * Remember to release the returned vout_thread_t.
@@ -628,3 +629,340 @@ void libvlc_video_set_deinterlace( libvlc_media_player_t *p_mi, int b_enable,
 
     vlc_object_release( p_vout );
 }
+
+/*****************************************************************************
+ * libvlc_video_get_marq_option_as_int : get a marq option value
+ *****************************************************************************/
+int libvlc_video_get_marq_option_as_int( libvlc_media_player_t *p_mi,
+                                         libvlc_video_marq_option_t option,
+                                         libvlc_exception_t *p_e )
+{
+    int i_return, b_enable;
+    vout_thread_t *p_vout = GetVout( p_mi, p_e );
+
+    if( !p_vout )
+    {
+        libvlc_exception_raise( p_e, "Unable to get video output" );
+        return VLC_EGENERIC;
+    }
+
+    /* get marq video filter status */
+    char *psz_string = var_GetString( vout_GetSpu( p_vout ), "sub-filter" );
+    if( !psz_string ) psz_string = strdup("");
+    char *psz_parser = strstr( psz_string, "marq" );
+
+    if( psz_parser )
+        b_enable = true;
+    else
+        b_enable = false;
+
+    /* return marq video filter status */
+    if ( option == libvlc_marq_Enabled )
+        return b_enable;
+
+    if( !b_enable )
+    {
+        /* if marq is disabled don't get options values */
+        libvlc_exception_raise( p_e, "Marq video filter is currently disabled" );
+        return VLC_EGENERIC;
+    }
+
+    /* get the vlc_object_t corresponding to the option choosen */
+    vlc_object_t *p_marq_object = ( vlc_object_t * )
+        vlc_object_find_name( p_vout, "marq", FIND_CHILD );
+
+    if( !p_marq_object )
+    {
+        libvlc_exception_raise( p_e, "Unable to get marq object" );
+        return VLC_EGENERIC;
+    }
+
+    switch( option )
+    {
+        case libvlc_marq_Color:
+        {
+            /* get text color */
+            i_return = var_GetInteger(p_marq_object, "marq-color");
+            break;
+        }
+        case libvlc_marq_Opacity:
+        {
+            /* get text opacity */
+            i_return = var_GetInteger(p_marq_object, "marq-opacity");
+            break;
+        }
+        case libvlc_marq_Position:
+        {
+            /* get text position */
+            i_return = var_GetInteger(p_marq_object, "marq-position");
+            break;
+        }
+        case libvlc_marq_Size:
+        {
+            /* get text size */
+            i_return = var_GetInteger(p_marq_object, "marq-size");
+            break;
+        }
+        case libvlc_marq_Refresh:
+        {
+            /* get refresh value */
+            i_return = var_GetInteger(p_marq_object, "marq-refresh");
+            break;
+        }
+        case libvlc_marq_Timeout:
+        {
+            /* get timeout value */
+            i_return = var_GetInteger(p_marq_object, "marq-timeout");
+            break;
+        }
+        case libvlc_marq_X:
+        {
+            /* get abscissa position */
+            i_return = var_GetInteger(p_marq_object, "marq-x");
+            break;
+        }
+        case libvlc_marq_Y:
+        {
+            /* get ordinate position */
+            i_return = var_GetInteger(p_marq_object, "marq-y");
+            break;
+        }
+        case libvlc_marq_Error:
+        {
+            /* Error */
+            libvlc_exception_raise( p_e, "Unable to get a marq option" );
+            i_return = VLC_EGENERIC;
+            break;
+        }
+        default: break;
+    }
+
+    if ( option != libvlc_marq_Enabled )
+        vlc_object_release( p_marq_object );
+    vlc_object_release( p_vout );
+
+    return i_return;
+}
+
+/*****************************************************************************
+ * libvlc_video_get_marq_option_as_string : get string value
+ *****************************************************************************/
+char *libvlc_video_get_marq_option_as_string( libvlc_media_player_t *p_mi,
+                                              libvlc_video_marq_option_t option,
+                                              libvlc_exception_t *p_e )
+{
+    char *psz_text;
+    vout_thread_t *p_vout = GetVout( p_mi, p_e );
+
+    if( !p_vout )
+    {
+        libvlc_exception_raise( p_e, "Unable to get video output" );
+        return NULL;
+    }
+
+    int b_enable = libvlc_video_get_marq_option_as_int( p_mi, libvlc_marq_Enabled, p_e);
+
+    if( !b_enable )
+    {
+        /* if marq is disabled don't get options values */
+        libvlc_exception_raise( p_e, "Marq video filter is currently disabled" );
+        return NULL;
+    }
+
+    /* get the vlc_object_t corresponding to the option choosen */
+    vlc_object_t *p_marq_object = ( vlc_object_t * )
+        vlc_object_find_name( p_vout, "marq", FIND_CHILD );
+
+    if( !p_marq_object )
+    {
+        libvlc_exception_raise( p_e, "Unable to get marq object" );
+        return NULL;
+    }
+
+    switch( option )
+    {
+        case libvlc_marq_Text:
+        {
+            /* get string value */
+            psz_text = var_GetString(p_marq_object, "marq-marquee");
+            break;
+        }
+        case libvlc_marq_Error:
+        {
+            /* Error */
+            libvlc_exception_raise( p_e, "Unable to get a marq option" );
+            return NULL;
+            break;
+        }
+        default: break;
+    }
+
+    vlc_object_release( p_marq_object );
+    vlc_object_release( p_vout );
+
+    return psz_text;
+}
+
+/*****************************************************************************
+ * libvlc_video_set_marq_option_as_int : enable, disable or set an int option
+ *****************************************************************************/
+void libvlc_video_set_marq_option_as_int( libvlc_media_player_t *p_mi,
+                                          libvlc_video_marq_option_t option,
+                                          int i_val, libvlc_exception_t *p_e )
+{
+    vout_thread_t *p_vout = GetVout( p_mi, p_e );
+
+    if( !p_vout )
+    {
+        libvlc_exception_raise( p_e, "Unable to get video output" );
+        return;
+    }
+
+    if ( option == libvlc_marq_Enabled )
+    {
+        /* enable or disable marq video filter */
+        vout_EnableFilter(p_vout, "marq", i_val, false);
+        return;
+    }
+
+    int b_enable = libvlc_video_get_marq_option_as_int( p_mi, libvlc_marq_Enabled, p_e);
+
+    if( !b_enable )
+    {
+        /* if marq is disabled don't set options values */
+        libvlc_exception_raise( p_e, "Marq video filter is currently disabled" );
+        return;
+    }
+
+    /* get the vlc_object_t corresponding to the option choosen */
+    vlc_object_t *p_marq_object = ( vlc_object_t * )
+        vlc_object_find_name( p_vout, "marq", FIND_CHILD );
+
+    if( !p_marq_object )
+    {
+        libvlc_exception_raise( p_e, "Unable to get marq object" );
+        return;
+    }
+
+    /* set marq options */
+    switch( option )
+    {
+        case libvlc_marq_Color:
+        {
+            /* change text color */
+            var_SetInteger(p_marq_object, "marq-color", i_val);
+            break;
+        }
+        case libvlc_marq_Opacity:
+        {
+            /* change text opacity */
+            var_SetInteger(p_marq_object, "marq-opacity", i_val);
+            break;
+        }
+        case libvlc_marq_Position:
+        {
+            /* change text position */
+            var_SetInteger(p_marq_object, "marq-position", i_val);
+            break;
+        }
+        case libvlc_marq_Size:
+        {
+            /* change text size */
+            var_SetInteger(p_marq_object, "marq-size", i_val);
+            break;
+        }
+        case libvlc_marq_Refresh:
+        {
+            /* set refresh value */
+            var_SetInteger(p_marq_object, "marq-refresh", i_val);
+            break;
+        }
+        case libvlc_marq_Timeout:
+        {
+            /* set timeout value */
+            var_SetInteger(p_marq_object, "marq-timeout", i_val);
+            break;
+        }
+        case libvlc_marq_X:
+        {
+            /* change abscissa position */
+            var_SetInteger(p_marq_object, "marq-x", i_val);
+            break;
+        }
+        case libvlc_marq_Y:
+        {
+            /* change ordinate position */
+            var_SetInteger(p_marq_object, "marq-y", i_val);
+            break;
+        }
+        case libvlc_marq_Error:
+        {
+            /* Error */
+            libvlc_exception_raise( p_e, "Unable to set a marq option" );
+            break;
+        }
+        default: break;
+    }
+
+    if ( option != libvlc_marq_Enabled )
+        vlc_object_release( p_marq_object );
+    vlc_object_release( p_vout );
+}
+
+/*****************************************************************************
+ * libvlc_video_set_marq_option_as_string : set a string option
+ *****************************************************************************/
+void libvlc_video_set_marq_option_as_string( libvlc_media_player_t *p_mi,
+                                             libvlc_video_marq_option_t option,
+                                             const char *psz_text,
+                                             libvlc_exception_t *p_e )
+{
+    vout_thread_t *p_vout = GetVout( p_mi, p_e );
+
+    if( !p_vout )
+    {
+        libvlc_exception_raise( p_e, "Unable to get video output" );
+        return;
+    }
+
+    int b_enable = libvlc_video_get_marq_option_as_int( p_mi, libvlc_marq_Enabled, p_e);
+
+    if( !b_enable )
+    {
+        /* if marq is disabled don't set options values */
+        libvlc_exception_raise( p_e, "Marq video filter is currently disabled" );
+        return;
+    }
+
+    /* get the vlc_object_t corresponding to the option choosen */
+    vlc_object_t *p_marq_object = ( vlc_object_t * )
+        vlc_object_find_name( p_vout, "marq", FIND_CHILD );
+
+    if( !p_marq_object )
+    {
+        libvlc_exception_raise( p_e, "Unable to get marq object" );
+        return;
+    }
+
+    /* set marq options */
+    switch( option )
+    {
+        case libvlc_marq_Text:
+        {
+            /* set a new text */
+            if ( psz_text != NULL )
+                var_SetString(p_marq_object, "marq-marquee", psz_text);
+            break;
+        }
+        case libvlc_marq_Error:
+        {
+            /* Error */
+            libvlc_exception_raise( p_e, "Unable to set a marq option" );
+            break;
+        }
+        default: break;
+    }
+
+    vlc_object_release( p_marq_object );
+    vlc_object_release( p_vout );
+}
diff --git a/src/libvlc.sym b/src/libvlc.sym
index b8ae128..16af73e 100644
--- a/src/libvlc.sym
+++ b/src/libvlc.sym
@@ -179,6 +179,8 @@ libvlc_video_get_aspect_ratio
 libvlc_video_get_chapter_description
 libvlc_video_get_crop_geometry
 libvlc_video_get_height
+libvlc_video_get_marq_option_as_int
+libvlc_video_get_marq_option_as_string
 libvlc_video_get_scale
 libvlc_video_get_spu
 libvlc_video_get_spu_count
@@ -192,6 +194,8 @@ libvlc_video_get_width
 libvlc_video_set_aspect_ratio
 libvlc_video_set_crop_geometry
 libvlc_video_set_deinterlace
+libvlc_video_set_marq_option_as_int
+libvlc_video_set_marq_option_as_string
 libvlc_video_set_scale
 libvlc_video_set_spu
 libvlc_video_set_subtitle_file
-- 
1.5.4.3

